\\ Copyright 2022 Kevin Ryde

\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


\\ Usage: read("nthprimeweb.gp");
\\        prime = nthprimeweb_prime(10^10);
\\        num = nthprimeweb_primepi(9^9);
\\
\\ This is some code for reading and caching prime() or primepi() data from
\\ The Nth Prime Page by Andrew Booker,
\\
\\     https://primes.utm.edu/nthprime/index.php
\\
\\ This web page is an a web interface to prime calculations equivalent to
\\ PARI prime() and primepi().  At the time of writing, it goes up to
\\
\\     nthprimeweb_prime(10^12)      \\ maximum
\\     nthprimeweb_primepi(3*10^13)  \\ maximum
\\
\\ The downloading and caching work is done by a Perl script
\\ "nthprimeweb-fetch".
\\
\\
\\ Changes
\\ -------
\\
\\ Version 1 - the first version.

nthprimeweb_version = 1;

\\ Perl script which does actual data reading.
nthprimeweb_INTERNAL_program = "nthprimeweb-fetch";

{
\\ if not already set by the user
if(nthprimeweb_verbose=='nthprimeweb_verbose,
nthprimeweb_verbose = 0
);
}

\\-----------------------------------------------------------------------------

\\ program run like
\\     nthprimeweb-fetch --gp \
\\                       --check-version 1 \
\\                       --prime 12345
\\                      
\\
nthprimeweb_INTERNAL_run(table,n) =
{
  my(str=Str(nthprimeweb_INTERNAL_program,
             " --check-version ",nthprimeweb_version
             " --",table," ",n));
  if(nthprimeweb_verbose, print(str));
  my(value=extern(str));
  type(value) == "t_INT" || error("oops, result not a number");
  value;
}

nthprimeweb_prime(n) = nthprimeweb_INTERNAL_run("prime",n);
{
addhelp(nthprimeweb_prime,
"prime = nthprimeweb_prime(n)
Return the n'th prime.
Like prime(n) but getting data from the nthprime web site.

At the time of writing, the web site goes up to n = 10^12.  A GP error()
results on exceeding this bound or any other problem.");
}

nthprimeweb_primepi(n) = nthprimeweb_INTERNAL_run("pi",n);
{
addhelp(nthprimeweb_primepi,
"num = nthprimeweb_primepi(n)
Return the number of primes <= n.
Like primepi(n) but getting data from the nthprime web site.

At the time of writing, the web site goes up to n = 3*10^13.  A GP error()
results on exceeding this bound, or any other problem.");

\\-----------------------------------------------------------------------------
